% function [out_y, out_pars] = kv_phase_optimize(in_y, in_opt, ...)
% Function optimizes the phase of first dimension of complex array in_y.  
%   in_opt.phase_algorithm   - algorithm of phase optimization
%        'manual_zero_order' - rotate on angle in_opt.phase_zero_order
%        'max_real_single'   - indepenent, trace by trace
%        'max_real_all'      - use one phase for all slices 
%   ...                      - parameter-value comma separated pairs
%   out_pars                 - structure of supplementary information

% boep, 15/09/2006

function [out_y, out_pars] = kv_phase_optimize(in_y, in_opt, vargin)

if nargin == 2
elseif nargin > 2
    if ~mod(nargin-1,2)
        for kk=1:2:nargin-1
            in_opt=setfield(in_opt, lower(varargin{kk}), varargin{kk+1});    
        end
    else, error('kv_phase_optimize: Wrong amount of arguments')
    end
else, error('Usage: [out_y, out_pars] = kv_phase_optimize(in_y, in_opt, vargin)');
end

out_pars = in_opt;

echo_select_algorithm = lower(safeget(in_opt, 'echo_select_algorithm', 'unknown'));
phase_algorithm = lower(safeget(in_opt, 'phase_algorithm', 'unknown'));

fopt=optimset('MaxFunEvals',200);

sz    = size(in_y);
sz2   = prod(sz(2:end));
yy = reshape(in_y, [sz(1), sz2]);
out_pars.is_performed = false;

[yyy, idx] = max(abs(yy));
out_pars.max_idx_all = idx;
% use_idx(1:sz2,1) = in_opt.max_idx;
switch echo_select_algorithm
    case 'manual'
        out_pars.max_idx = in_opt.echo_idx;
%         use_idx(1:) = in_opt.max_idx;
    case 'statistics'
        % find statistically most reasonable phase and zero point
        if length(idx) > 1
            miny = floor(min(min(idx)) + 0.5);
            maxy = floor(max(max(idx)) + 0.5);
            xx = (miny:maxy);
            xx = [xx(1)-0.5 xx + 0.5];
            xx(1) = min(xx(1),miny);
            xx(end) = max(xx(end),maxy);
            nn = histc(idx,xx);
            [mmm, iidx] = max(nn);
            out_pars.max_idx = floor(xx(iidx)+0.5);
        else
            out_pars.max_idx = idx;
        end
    case 'single'
        out_pars.max_idx = idx(1);
        l_idx = size(yy,1);
        for ii=1:length(idx)
            shift = idx(1)-idx(ii);
            yy(:,ii) = [zeros(max(0,shift),1);yy(max(1,1+shift):l_idx+min(shift,0),ii);zeros(max(0,-shift),1);];
        end
end

switch phase_algorithm
 % manual rotation of the phase
    case 'manual_zero_order'
        yy = phase_zero_order(yy, in_opt.phase_zero_order);
        out_pars.phase_zero_order = in_opt.phase_zero_order;
        out_pars.max_idx = max(real(yy));
        out_pars.is_performed = true;
 % find maximum of the every data slice
 % change phase to make make maximum at phase 0
    case 'max_real_single'
        [yyy, idx] = max(abs(yy));
        out_pars.max_idx_all = idx';
        for ii=1:sz2
            val = yy(idx(ii),ii);
            out_pars.phase_zero_order(ii,1) = ...
                atan2(imag(val), real(val));
            yy(:,ii) = phase_zero_order(yy(:,ii), out_pars.phase_zero_order(ii,1));
        end
        out_pars.max_idx = floor(sum(idx)/length(idx) + 0.5);
        out_pars.is_performed = true;
% find maximum of the every data slice
% calculate everage maximum point and average angle
% rotate on the calculated angle
    case 'max_real_all'
        for ii=1:sz2
            val = yy(idx(ii),ii);
            phase(ii,1) = atan2(imag(val), real(val));
        end
        out_pars.phase_zero_order = sum(phase)/length(phase);
        yy = phase_zero_order(yy, out_pars.phase_zero_order);
        out_pars.is_performed = true;
    case 'max_real_manual'
        [yyy, out_pars.max_idx_all] = max(abs(yy));
        out_pars.max_idx     = in_opt.max_idx;
        for ii=1:sz2
            val = yy(in_opt.max_idx,ii);
            phase(ii,1) = atan2(imag(val), real(val));
            yy(:,ii) = phase_zero_order(yy(:,ii), phase(ii,1));
        end
        out_pars.phase_zero_order = phase;
        out_pars.is_performed = true;
    case 'min_imag_manual'
        [yyy, out_pars.max_idx_all] = max(abs(yy));
        out_pars.max_idx     = in_opt.max_idx;
        % find symmetric interval around echo maximum
        echo_spread = min(abs(in_opt.max_idx - min(in_opt.echo_area)), abs(in_opt.max_idx - max(in_opt.echo_area)));
        echo_area   = in_opt.max_idx-echo_spread : in_opt.max_idx+echo_spread;
        ph = 0;
        opt = optimset('MaxIter', 500);
        for ii=1:sz2
           ph = fminsearch(@optimize_min_imag_manual, ph, opt, yy(echo_area,ii));
           yy(:,ii) = phase_zero_order(yy(:,ii), ph);
           [mmm, idx] = max(abs(real(yy(:,ii))));
           if real(yy(idx,ii)) < 0
               ph = ph + pi;
               yy(:,ii) = phase_zero_order(yy(:,ii), pi);
           end
           phase(ii,1) = ph;
        end
        out_pars.phase_zero_order = phase;
        out_pars.is_performed = true;
    otherwise
        disp('No phase optimization was performed.');
end
out_y = reshape(yy, sz);

% -------------------------------------------------------------------------
function out_y = phase_zero_order(in_y, in_phase)
out_y = in_y.*exp(-i*in_phase);

function out_res = optimize_min_imag_manual(in_par, in_y)
out_res = abs(sum(imag(phase_zero_order(in_y, in_par(1)))));
